(***********************************************************************)
(*                                                                     *)
(*                                GALAX                                *)
(*                             XQuery Engine                           *)
(*                                                                     *)
(*  Copyright 2001-2003, Lucent Technologies.                          *)
(*  Distributed only by permission.                                    *)
(*                                                                     *)
(***********************************************************************)

(* $Id: xquery_ast_util.mli,v 1.22 2006/01/23 19:22:44 simeon Exp $ *)

(* Module: Xquery_astutil
   Description:
     This module implements some useful operations the XQuery AST.
*)

open Xquery_common_ast
open Xquery_type_ast
open Xquery_ast
open Xquery_algebra_ast


(**************************)
(* AST creation functions *)
(**************************)

(* Sequence type AST creation *)

val mksequencetype : sequencetype_desc -> sequencetype
val fmksequencetype : sequencetype_desc -> Finfo.finfo -> sequencetype

(* Typeswitch pattern AST creation *)

val mkpattern : pattern_desc -> pattern
val fmkpattern : pattern_desc -> Finfo.finfo -> pattern

(* XQuery AST creation *)

val mkexpr : expr_desc -> expr
val fmkexpr : expr_desc -> Finfo.finfo -> expr

val mkstep_qualifier : step_qualifier_desc -> step_qualifier
val fmkstep_qualifier : step_qualifier_desc -> Finfo.finfo -> step_qualifier

val mkfl_expr : fl_expr_desc -> fl_expr
val fmkfl_expr : fl_expr_desc -> Finfo.finfo -> fl_expr

(* XQuery prolog AST creation *)

val mkfunction_def : function_def_desc -> function_def
val fmkfunction_def : function_def_desc -> Finfo.finfo -> function_def

val mkvar_decl : var_decl_desc -> var_decl
val fmkvar_decl : var_decl_desc -> Finfo.finfo -> var_decl

val mkindex_def : index_def_desc -> index_def
val fmkindex_def : index_def_desc -> Finfo.finfo -> index_def

val mkcontext_decl : context_decl_desc -> context_decl
val fmkcontext_decl : context_decl_desc -> Finfo.finfo -> context_decl


(**************************)
(* AST accessor functions *)
(**************************)

(* Processes NS attributes of an element *)

val get_ns_attributes               : expr list -> ((Namespace_names.prefix * Namespace_names.uri) list * expr list)


(*****************************)
(* Module related operations *)
(*****************************)

(* The empty module *)

val empty_prolog : prolog

(* Merging two modules *)

val merge_library_module_in_prolog : Namespace_names.uri -> prolog -> library_module -> prolog
val merge_prologs_with_decls : context_decl list -> prolog -> prolog -> prolog

(* Extracting the statements from a given module *)

val split_main_module : main_module -> prolog * statement list

(* Remove boundary whitespace, in case XQuery whitespace handling is
   set to 'strip' *)

val remove_boundary_whitespace_from_children : expr list -> expr list

