(***********************************************************************)
(*                                                                     *)
(*                                 GALAX                               *)
(*                              XQuery Engine                          *)
(*                                                                     *)
(*  Copyright 2001-2004, Lucent Technologies and AT&T Labs.            *)
(*  Distributed only by permission.                                    *)
(*                                                                     *)
(***********************************************************************)

(* $Id: algebra_type.mli,v 1.7 2006/02/02 21:03:16 mff Exp $ *)

(* Module Xquery_algebra_type
   Description:
     This *interface* contains type declarations for evaluation code.
*)

open Dm

open Physical_value

open Xquery_algebra_ast
open Xquery_common_ast
open Execution_context

open Alg_path_struct


(******************)
(* Execution code *)
(******************)

type physical_annotation =
    { has_nested_snap   : bool;
      has_side_effect   : bool;
      has_delta_update  : bool;
      path_annotation   : path_annotation}

type eval_code_unit   = algebra_context -> unit -> physical_value
type eval_code_unary  = algebra_context -> physical_value -> physical_value
type eval_code_binary = algebra_context -> physical_value -> physical_value -> physical_value
type eval_code_many   = algebra_context -> physical_value array -> physical_value

type alg_eval_code =
  | AOECUnit of eval_code_unit
  | AOECUnary of eval_code_unary
  | AOECBinary of eval_code_binary
  | AOECMany of eval_code_many

(* Type for execution code *)

type eval_fun =
    algebra_context -> algop_expr -> physical_value
and alg_eval_code_dep =
  | NoDep of alg_eval_code
  | SomeDep of (eval_fun -> alg_eval_code) 
and algop_sub_exprs = (alg_eval_code_dep, physical_annotation) aalgop_sub_exprs
and algop_expr = (alg_eval_code_dep, physical_annotation) aalgop_expr


(***************************************)
(* Execution code for the query prolog *)
(***************************************)

type eval_code_unit_prolog =
    algebra_context -> unit -> algebra_context
type eval_code_unary_prolog =
    algebra_context -> physical_value -> algebra_context
type eval_code_binary_prolog =
    algebra_context -> physical_value -> physical_value -> algebra_context
type eval_code_many_prolog =
    algebra_context -> physical_value array -> algebra_context

type alg_eval_code_prolog =
  | PAOECUnit of eval_code_unit_prolog
  | PAOECUnary of eval_code_unary_prolog
  | PAOECBinary of eval_code_binary_prolog
  | PAOECMany of eval_code_many_prolog
and alg_eval_code_dep_prolog =
  | PNoDep of alg_eval_code_prolog
  | PSomeDep of (eval_fun -> alg_eval_code_prolog) 

(* Type for execution code *)

type eval_fun_prolog =
    algebra_context -> algop_expr -> algebra_context

type algop_decl =
    (alg_eval_code_dep, physical_annotation, alg_eval_code_dep_prolog) aalgop_decl
type algop_prolog =
    (alg_eval_code_dep, physical_annotation, alg_eval_code_dep_prolog) aalgop_prolog
type algop_xmodule =
    (alg_eval_code_dep, physical_annotation, alg_eval_code_dep_prolog) aalgop_xmodule
type algop_function_body =
    (alg_eval_code_dep, physical_annotation) aalgop_expr
type alg_compile_context =
    (alg_eval_code_dep, physical_annotation) Compile_context.compile_context


(* Physical signatures *)

(* Physical: [Cursor(item); ... Cursor(item)] -> Cursor(item) *)
type item_cursor_nary_to_item_cursor_code =
    algebra_context -> Physical_value.item Cursor.cursor array -> Physical_value.item Cursor.cursor

type item_list_nary_to_item_list_code =
    algebra_context -> Physical_value.item list array -> Physical_value.item list

