open Code_selection_context
open Compile_context
open Cs_util_coercion
open Galax_server
open Error
open Execution_context
open Physical_item
open Physical_value
open Xquery_algebra_ast
open Xquery_algebra_ast_util

(* CEExecute *)
let build_default_ceexecute_code code_ctxt algop_remote =
  let comp_ctxt = compile_context_from_code_selection_context code_ctxt in
  let proc_ctxt = processing_context_from_compile_context comp_ctxt in
  let buf = Buffer.create 8192 in 
  (* 
     Before we serialize the plan, we identify all the free variables ($x_k) in the remote plan, 
     extract the values ($v_k) for those variables from the execution context, then serialize the
     following closed plan in which no variables are free:

     let $x_1 := v_1, ..., $x_k := v_k
     return remote_plan

     let v = (build_current_retrieve_code code_ctxt x) () in
     logical_aalgop_mkop (AOELet (None, x)) indep dep None eh fi 

     One problem is that we do not have a representation for Literal
     item values that does not require constructing
     an algebraic expression that _computes_ that value.  Also, we
     want to include the type of a literal value, so that when it is
     parsed on the server side, its dynamic type is correct.

  *)
  let _ = Planio_print_xml.serialize_logical_algebra_statement proc_ctxt (Galax_io.Buffer_Output buf) algop_remote in
  let plan = Buffer.contents buf in
  (fun alg_ctxt hostpv portpv ->
    let host = Physical_util.get_string hostpv in 
    let port = Decimal._int_of_integer(Physical_util.get_integer portpv) in  
(* Galax expects an integer, but the server wants a string *)
    let res = Server.do_query(host, Some (string_of_int port), XQueryPlan, plan) in 
  (*
     We could return a token stream directly, but first we need a way
     to register the physical-type signature of Execute
     C.f. Code_builtin_fn.select_physical_op and
     Code_builtin_fn.build_builtin_fn_code.
  *)
      let (result_f,result_ch) = Filename.open_temp_file "res" ".xml" in
      output_string result_ch res;
      close_out result_ch;

  (* Write result to a file, register the file, and re-parse. *)
      let result = Fn_doc.lookup_doc_function_no_table result_f proc_ctxt in
      Sys.remove result_f;

  (* The result will be well-formed document, i.e., a document
     containing a value boxed in a <glx:result> element or
     <glx:error>.  We have to unbox it here.  
  *)
      let doc = (Physical_item.getNode(List.hd result))#getDocumentNode() in 
      let rootelem = Cursor.cursor_get_singleton(doc#children None) in
      let rootelem_qname = 
	match rootelem#node_name() with
	| Some q -> Namespace_symbols.relem_name(q#getAtomicQName())
	| None -> raise (Query(Error("DXQ Server: Root element of return value has no name")))
      in
      if (rootelem_qname = Namespace_builtin.glx_error) then 
	raise (Query(Error("DXQ Server: "^(rootelem#string_value()))))
      else
	(* We still need to unbox the <glx:result> value and construct a
           sequence of typed items
         *) 
	Cursor.cursor_of_singleton(Physical_value.Item_Node rootelem)
  )

(* What should be the physical type of the execute operator? *)
let build_execute_code code_ctxt algop =

  (* AOEExecute takes three independent sub-expressions: the host, the
     port, and the plan to execute remotely *)

  let indep = access_manysub algop.psub_expression in
  let (hostao,portao,algop_remote) = 
    (Array.get indep 0,Array.get indep 1,Array.get indep 2)
  in
  let fn = build_default_ceexecute_code code_ctxt algop_remote in
  (* 
     After we have serialized the plan to execute remotely, the
     independent subexpression of the remote expression is removed
     from the local plan.

     The effect is that the physical plan for the body of the
     "execute" expression is never executed locally.

     I considered an alternative, in which the remote plan was a
     dependent expression, but this would have required hacking
     Cs_code_selection_expr.default_code_selection to be aware of
     side-effects in this function.
  *)
  algop.psub_expression <- TwoSub(hostao, portao);
  (coerce_nodep fn coerce_binary_item_cursor_to_item_cursor), code_ctxt

