open Code_selection_context

open Cs_util_coercion
open Cs_util

open Xquery_common_ast
open Xquery_algebra_ast
open Xquery_algebra_ast_util
open Xquery_physical_algebra_ast
open Xquery_physical_type_ast

open Error


(* CEIf *)
let build_default_if_code code_ctxt =
  (fun (d1, d2) eval alg_ctxt pv  ->
     if Physical_util.get_boolean pv then
       eval alg_ctxt d1
     else eval alg_ctxt d2)

let build_if_code code_ctxt algop = 
  let fn = build_default_if_code code_ctxt in
  let dep = access_twosub algop.pdep_sub_expression in 
  (coerce_twodep fn dep coerce_unary_item_cursor_to_physical_value), code_ctxt

(* CELet *)
let build_default_let_code bind_fun =
  (fun e2 eval alg_ctxt input ->	   
     bind_fun input;
     eval alg_ctxt e2
  )

let build_default_let_sax_value_code code_ctxt v =
  build_default_let_code (Code_binding.build_bind_sax_value_to_variable_code code_ctxt v)

let build_default_let_sax_discard_code code_ctxt v =
  build_default_let_code (fun s -> Streaming_ops.discard_typed_xml_stream s)

let build_default_let_item_cursor_code code_ctxt v =
  build_default_let_code (Code_binding.build_bind_item_cursor_to_variable_code code_ctxt v)

let build_default_let_type_checked_item_cursor_code code_ctxt dt v =
  build_default_let_code (Code_binding.build_bind_type_checked_item_cursor_to_variable_code code_ctxt dt v)

let build_default_let_item_list_code code_ctxt v =
  build_default_let_code (Code_binding.build_bind_item_list_to_variable_code code_ctxt v)

let build_default_let_type_checked_item_list_code code_ctxt dt v =
  build_default_let_code (Code_binding.build_bind_type_checked_item_list_to_variable_code code_ctxt dt v)

let build_let_code_physical_type code_ctxt algop (odt, vn) = 
  let dep = access_onesub algop.pdep_sub_expression in 
  let csc = add_variable_to_current_context code_ctxt vn in
  let physop = Cs_util.get_physical_opname algop in 
    match physop with
      | POLet (_, PT_Dom PT_CursorSeq) ->
	  begin
	    match odt with
	      | Some dt ->
		  let fn  = build_default_let_type_checked_item_cursor_code csc dt vn in 
		    (coerce_onedep fn dep coerce_unary_item_cursor_to_physical_value), csc
	      | _ ->
		  let fn  = build_default_let_item_cursor_code csc vn in 
		    (coerce_onedep fn dep coerce_unary_item_cursor_to_physical_value), csc
	  end
      | POLet (_, PT_Dom PT_ListSeq) ->
	  begin
	    match odt with
	      | Some dt ->
		  let fn  = build_default_let_type_checked_item_list_code csc dt vn in
		    (coerce_onedep fn dep coerce_unary_item_cursor_to_physical_value), csc
	      | _ ->
		  let fn  = build_default_let_item_list_code csc vn in
		    (coerce_onedep fn dep coerce_unary_item_list_to_physical_value), csc
	  end
      | POLet (_, PT_Sax PT_Stream) ->
	  let fn  = build_default_let_sax_value_code csc vn in 
	    (coerce_onedep fn dep coerce_unary_sax_to_physical_value), csc
      | POLet (_, PT_Sax PT_Discarded) ->
	  let fn  = build_default_let_sax_discard_code csc vn in 
	    (coerce_onedep fn dep coerce_unary_sax_to_physical_value), csc
      | _ -> raise(Query(Code_Selection("Invalid physical operator in build_let_code")))

let build_default_let_code_old code_ctxt odt v =
  (* Now binding an xml_value here. - Michael *)
  let stat_ctxt = static_context_from_code_selection_context code_ctxt in
  let add_fn    = Cs_util.build_add_var_xml_value_unsafe_allowed code_ctxt v in
    match odt with
      | None -> 
	  (fun e2 eval alg_ctxt input_cursor ->	   
	     add_fn input_cursor;
	     eval alg_ctxt e2
	  )
	    
      | Some dt -> 
	  (fun e2 eval alg_ctxt input_cursor ->
	     (* For the dynamic type check, we fall back to item cursors for the moment. *)
	     let item_cursor_of_xml_value x = Physical_value_util.item_cursor_of_physical_value (Physical_value_util.physical_value_of_xml_value x) in
	     let xml_value_of_item_cursor x = Physical_value_util.xml_value_of_physical_value (Physical_value_util.physical_value_of_item_cursor x) in
	     let type_checked_cursor = Code_util_matching.dynamic_type_check stat_ctxt dt (item_cursor_of_xml_value input_cursor) in 
	       add_fn (xml_value_of_item_cursor type_checked_cursor);
	       eval alg_ctxt e2
	  )

let build_let_code_old code_ctxt algop (odt, vn) = 
  let dep = access_onesub algop.pdep_sub_expression in 
  let csc = add_variable_to_current_context code_ctxt vn in
  let fn  = build_default_let_code_old csc odt vn in 
    (coerce_onedep fn dep coerce_unary_xml_to_physical_value), csc

(* Exposed *)
let build_let_code code_ctxt algop (odt, vn) =
  if (!Conf.code_selection_by_physical_type)
  then build_let_code_physical_type code_ctxt algop (odt, vn)
  else build_let_code_old code_ctxt algop (odt, vn)

(* CEVar *)
let build_default_cevar_code code_ctxt vn =
  let retrieve_code = build_physical_value_retrieve code_ctxt vn in
  (fun alg_ctxt () -> retrieve_code () )
  
let build_var_code code_ctxt algop vn = 
  let fn = build_default_cevar_code code_ctxt vn in
  let _ = access_nosub algop.pdep_sub_expression in 
  (coerce_nodep fn coerce_unit_to_physical_value), code_ctxt
