open Compile_context
open Cs_util_coercion
open Code_selection_context
open Code_util_matching
open Namespace_names
open Error
open Physical_value_util
open Xquery_algebra_ast
open Xquery_algebra_ast_util

(* User defined wrapper for compiled functions *)
let build_default_user_defined_function code_ctxt fn_body_ref fn vars optintypes optdtm =
  if (Array.length optintypes) != (Array.length vars)
  then
    raise (Query (Wrong_Args ("Number of types and variables does not match for user defined function "
			      ^ (prefixed_string_of_rqname fn))));
  let arity = (Array.length optintypes) in
  let stat_ctxt = static_context_from_code_selection_context code_ctxt in
  let fn_code_block = 
    Code_selection_context.build_function_code code_ctxt (fn,arity) vars in

  let insert_code_array = fn_code_block.parameter_insertion_code in
  let enter_context     = fn_code_block.entrance_code  in
  let exit_context      = fn_code_block.exit_code      in

  (fun () eval alg_ctxt the_args -> 
  if arity != (Array.length the_args)
  then
    raise (Query (Wrong_Args ("Incompatible number of elements in function"
			      ^ (prefixed_string_of_rqname fn))));

    (* Enter the functions context *)
    enter_context ();
    (* Perform the actual insert *)
    for i = 0 to arity - 1 do	   
      insert_code_array.(i) 
	(materialize_physical_value (physical_value_of_item_cursor (
				     (dynamic_opttype_check stat_ctxt optintypes.(i) the_args.(i)))))
    done;
    (* Type match return value and return type *)
    let return_expr  = eval alg_ctxt !fn_body_ref in
    let return_value = 
      materialize_physical_value (physical_value_of_item_cursor
				    (dynamic_opttype_check stat_ctxt (Some optdtm) 
				       (item_cursor_of_physical_value return_expr))) in
    (* Exit the context *)
    exit_context ();
    return_value
  )

let build_user_defined_fn_code code_ctxt algop ((fname,arity), optintypes, outtype) = 
	      (* Bind to the function *)	
  let comp_ctxt = annotated_compile_context_from_code_selection_context code_ctxt in
  let vars,fn_aops_ref,_ = get_function_from_compile_context comp_ctxt (fname,arity) in	  
  let dep = access_unitsub algop.pdep_sub_expression in 
  let fn = build_default_user_defined_function code_ctxt fn_aops_ref
      fname vars optintypes outtype in
  (coerce_unitdep fn dep coerce_many_item_cursor_to_physical_value), code_ctxt

