(***********************************************************************)
(*                                                                     *)
(*                                 GALAX                               *)
(*                              XQuery Engine                          *)
(*                                                                     *)
(*  Copyright 2001-2004, Lucent Technologies and AT&T Labs.            *)
(*  Distributed only by permission.                                    *)
(*                                                                     *)
(***********************************************************************)

(* $Id: cs_code_top.ml,v 1.16 2006/02/02 21:03:16 mff Exp $ *)

(* Module: Cs_top_code
   Description:
     This module contains the actual caml code that is used for
     evaluation of the prolog part of the algebra.
*)

open Error
open Namespace_names

open Xquery_core_ast
open Xquery_common_ast
open Algebra_type

open Norm_context
open Typing_context
open Compile_context
open Code_selection_context
open Execution_context

open Physical_value
open Physical_item
open Physical_sequence
open Physical_value_util 
open Physical_table

open Cs_util


(* Global variable declaration *)

let build_default_var_decl_code code_ctxt odt vn =
  let add_fn = build_add_var_item_cursor_safe code_ctxt vn in
  let stat_ctxt = static_context_from_code_selection_context code_ctxt in
  (fun  alg_ctxt input_cursor -> 
    let ndt = Code_util_matching.dynamic_opttype_check stat_ctxt odt input_cursor in
      (*add_var_item_cursor_to_algebra_context alg_ctxt (vn,ndt)*)
      add_fn ndt;
      alg_ctxt
  )
(* The array will not yet be allocated. So we must wait until execution to 
   store the value in the context. However we can do the type checking stuff 
   now.
*)
let build_default_var_decl_external_code code_ctxt odt vn=
  let stat_ctxt = static_context_from_code_selection_context code_ctxt in
  let fn_value  = get_external_variable_fn_value code_ctxt vn in
  let add_fn = build_add_var_item_cursor_safe code_ctxt vn in

  (fun alg_ctxt () ->
    let value = !fn_value () in
    let n = Cursor.cursor_of_list value in 
    let ndt = Code_util_matching.dynamic_opttype_check stat_ctxt odt n in       
       (* construct the fn. *)
    add_fn ndt;
    alg_ctxt
  )


(* Key declarations *)

let build_default_key_decl_code code_ctxt kn =
  let add_fn =
    build_add_var_xml_value_safe code_ctxt Xquery_common_ast.fs_dot
(* BUG: The 'unsafe' versions do not seem to work in the prolog! - Jerome
    build_add_var_xml_value_unsafe_allowed code_ctxt Xquery_common_ast.fs_dot *)
  in
  (fun ea2 eval alg_ctxt n1 ->
    let process_e2 alg_ctxt n1 =
      let n2 =
	(* Extends the dynamic environment for '.' *)
	add_fn (Physical_value.DomValue (sequence_of_singleton n1));
	(* Evaluates the key expression *)
	dom_value_of_physical_value (eval alg_ctxt ea2)
      in
      let keyval =
	try
	  if (sequence_is_empty n2) then 
	    ""
	  else
	    (* Mary ERROR HANDLING : This is a hack! *)
	    let i = (sequence_get_singleton n2) in
	    match item_kind i with 
	    | NodeKind -> (getNode i)#string_value()
	    | AtomicValueKind -> (getAtomicValue i)#erase_atomic_value()
	with
	| _ ->
	    raise (Query (Code_Selection "key definition: key value not an item"))
      in
      add_key_to_algebra_context alg_ctxt (kn,keyval) [n1]
    in
    Cursor.cursor_fold_left process_e2 alg_ctxt n1)

let build_default_name_index_decl_code code_ctxt =
  (fun alg_ctxt () ->
    alg_ctxt
  )

let build_default_twig_index_decl_code code_ctxt =
 (fun alg_ctxt () ->
    alg_ctxt
  )

