(***********************************************************************)
(*                                                                     *)
(*                                 GALAX                               *)
(*                              XQuery Engine                          *)
(*                                                                     *)
(*  Copyright 2001-2003, Lucent Technologies.                          *)
(*  Distributed only by permission.                                    *)
(*                                                                     *)
(***********************************************************************)

(* $Id: variable_context_manager.ml,v 1.6 2006/02/15 22:47:03 simeon Exp $ *)

(* This module eals with variable context during runtime. 
   It also holds the build functions to access and store variables *)

open Error
open Xquery_common_ast

open Namespace_names
open Physical_value
open Physical_value_util
open Array

(********************************)
(*** Variable Context Manager ***)
(********************************)


let empty_physical_value = physical_value_of_item_list []
(* Consider using polymorphic types to close off after instantiated objects *)

type variable_context_manager = {
  variable_stack            : physical_value array Stack.t;
  mutable current_variables : physical_value array ;
  last_slot                 : int ref; 
  (* Debugging parameters *)
  id                        : int;
  
}
type variable_ref = variable_context_manager * int

let string_of_variable_ref v = 
  let cm, vr = v in
    (string_of_int cm.id) ^ ":" ^ (string_of_int vr)

let get_new_variable_slot cm = 
  let ls = !(cm.last_slot) in
    incr (cm.last_slot);
    cm,ls

let debug_id = ref 0;;
(* We don't know the size at the outset *)
let build_context_manager () = 
  incr debug_id;
  { variable_stack = Stack.create ();
    current_variables = Array.create 1 empty_physical_value;
    last_slot = ref 0;
    id = !debug_id }

let instantiate_variable_context_manager cm = 
(*  print_endline ("Instantiating  context " ^ (string_of_int cm.id) ^ " with " ^ (string_of_int !(cm.last_slot)));  *)
  cm.current_variables <- Array.create !(cm.last_slot) empty_physical_value

let build_variable_enter_context cm = 
  (fun () ->
     Stack.push (Array.copy cm.current_variables) cm.variable_stack;
  )

let build_variable_exit_context cm = 
  (fun () ->
     try       
       let old_context = Stack.pop cm.variable_stack in
	 Array.blit old_context 0 cm.current_variables 0 !(cm.last_slot) 
     with Stack.Empty ->
       raise (Query (Code_Selection ("Tried to exit an empty context")))
  )


(* Insert and retrieve and code *)      
let build_parameter_insert_code cm vr =
  (fun pv -> cm.current_variables.(vr) <- pv)

let build_variable_store_code (var_ref:variable_ref) = 
  let cm, vr = var_ref in
  (fun pv -> cm.current_variables.(vr) <- pv)

let build_variable_retrieve_code (var_ref:variable_ref) = 
  let cm, vr = var_ref in
    (fun () -> cm.current_variables.(vr) )

