/***********************************************************************/
/*                                                                     */
/*                                 GALAX                               */
/*                              XQuery Engine                          */
/*                                                                     */
/*  Copyright 2001-2003, Lucent Technologies.                          */
/*  Distributed only by permission.                                    */
/*                                                                     */
/***********************************************************************/

/* $Id: example.c,v 1.7 2006/01/19 21:27:54 mff Exp $ */

/* A sample C client for the Caml functions */

#include <stdio.h>
#include "galax.h"


#define exit_on_error(_Expr) { \
  err = _Expr; \
  if (err != 0) {printf("%s\n", galax_error_string); exit(err);} \
} 

/* 
   Example 1 

   Build a query context from a file.
   Load a document.
   Add a global variable to query context, bound to the loaded document.
   Evaluate a query contained in a string.
   Serialize result to standard output

*/

void example1(processing_context pc, prepared_prolog pp)
{
  itemlist res;
  galax_err err; 
  char *query_input = "for $s in $report//section[section.title = \"Procedure\"] return ($s//incision)[2]/instrument";

  exit_on_error(galax_eval_statement(pp, Buffer_Input, query_input, &res));
  exit_on_error(galax_serialize_to_stdout(pc, res));
  itemlist_free(res);
}

/* 
   Example 2

   Build a query context from a file.
   Load a document.
   Evaluate a query contained in a file.
   Serialize result to standard output.

*/
void example2(processing_context pc,compiled_prolog cp)
{
  compiled_prolog cp1;
  prepared_prolog pp;
  compiled_module cm;
  compiled_stmt_list stmt_list;
  compiled_statement stmt;
  itemlist res;
  galax_err err;
  external_context exc;

  /*  exit_on_error(galax_import_main_module(cp, NoExternalContextItem, File_Input, "../docs/seq.xq", &cp1, &stmt_list));  */
  exit_on_error(galax_import_main_module(cp, NoExternalContextItem, File_Input, "../docs/seq.xq", &cm)); 
  exit_on_error(galax_default_external_context(&exc));
  exit_on_error(galax_eval_prolog(cm->compiled_prolog, exc, &pp));
  stmt_list = cm->compiled_stmts;
  while (!(is_empty(stmt_list))) { 
    stmt = items_first(stmt_list);
    stmt_list = items_next(stmt_list);
    exit_on_error(galax_eval_compiled_statement(pp, stmt, &res));
    exit_on_error(galax_serialize_to_stdout(pc,res));
    itemlist_free(res);
  }
}

/* 
   Example 3

   Build a query context from a file.
   Create some XML values. 
   Evaluate a query function contained in the query context by
   applying to XML values.
   Serialize result to a string & print. 

*/
void example3 (processing_context pc,compiled_prolog cp, itemlist xmldoc) 
{
  compiled_prolog cp1,cp2;
  prepared_prolog pp;
  external_context exc;
  compiled_stmt_list stmt_list;
  compiled_statement stmt;
  compiled_module cm;

  itemlist res;
  atomicString xmlstr;
  atomicInteger xmlint;
  galax_err err;
  char *vars[3];
  itemlist args[3];

  exit_on_error(galax_import_main_module(cp, NoExternalContextItem, File_Input,"../docs/xq-example2.xq", &cm));

  exit_on_error(galax_atomicString("a string", &xmlstr)); 
  exit_on_error(galax_atomicInteger(12, &xmlint));

  /* external context */
  vars[0] = "x";
  args[0] = itemlist_cons(xmlstr, NULL);
  vars[1] = "y"; 
  args[1] = itemlist_cons(xmlint, NULL); 
  vars[2] = "v";
  args[2] = itemlist_cons(xmlint, NULL);

  exit_on_error(galax_build_external_context (pc, itemlist_empty(), itemlist_empty(), vars, args, 3, &exc)); 
  exit_on_error(galax_eval_prolog(cm->compiled_prolog, exc, &pp)); 
  exit_on_error(galax_eval_statement(pp, Buffer_Input, "xq-example1:test2($y,$x)", &res));
  exit_on_error(galax_serialize_to_stdout(pc,res));
  itemlist_free(res);
}

void example4 (processing_context pc,prepared_prolog pp) { 
  itemlist res;
  galax_err err;

  exit_on_error(galax_eval_statement(pp, Buffer_Input, "<foo1:a xmlns:foo1='http://www.foo.org/'/>", &res));
  exit_on_error(galax_serialize_to_stdout(pc,res));
  itemlist_free(res);
}
int main(int argc, char ** argv)
{

  char *fake_argv[2];
  processing_context pc;
  itemlist xmldoc;
  compiled_prolog cp, cp1;
  prepared_prolog pp;
  galax_err err;
  external_context exc;

  //galax_set_sbdo_kind(pc, SBDO_DupTidy);
  //galax_set_projection_kind(pc, Projection_None);
  galax_init();

  exit_on_error(galax_default_processing_context(&pc));

  exit_on_error(galax_load_document(pc, File_Input, "../docs/report.xml", &xmldoc));
  exit_on_error(galax_load_standard_library(pc, &cp));
  exit_on_error(galax_import_library_module(cp, File_Input, "../docs/seq_context.xq", &cp1));
  exit_on_error(galax_default_external_context(&exc));
  exit_on_error(galax_eval_prolog(cp1, exc, &pp));
  
  example1(pc,pp);
  example2(pc,cp1);
  example3(pc,cp1, xmldoc); 
  example4(pc,pp);

  return 0;
}
