(***********************************************************************)
(*                                                                     *)
(*                                 GALAX                               *)
(*                              XQuery Engine                          *)
(*                                                                     *)
(*  Copyright 2001-2004, Lucent Technologies.                          *)
(*  Distributed only by permission.                                    *)
(*                                                                     *)
(***********************************************************************)

(* $Id: namespace_context.mli,v 1.2 2006/05/08 12:09:39 simeon Exp $ *)

(* Module: Namespace_context
   Description:
     This module implements support for the namespace context.
*)

open Namespace_names


(**********************)
(* Namespace bindings *)
(**********************)

(* Namespace binding *)

type binding = prefix * uri

(* Type for binding tables *)

type binding_table = binding list


(*************************)
(* Namespace environment *)
(*************************)

(* type for namespace environments *)

type nsenv

(* creates an empty environment *)

val empty_nsenv          : nsenv
val default_xquery_nsenv : nsenv
val default_xml_nsenv    : nsenv

(* adds a new prefix,uri binding in a namespace environment *)

val add_all_ns : nsenv -> binding_table -> nsenv

(* gets the namespace URI bound to a given prefix *)

val get_ns_of_prefix  : nsenv -> prefix -> uri

(* gets the closest namespace prefix bound to a given URI *)

val make_binding : nsenv -> rqname -> (uqname * binding option * binding)
val make_attribute_binding : nsenv -> rqname -> (uqname * binding option * binding)

(* Returns the set of current bindings *)

val active_bindings : nsenv -> binding_table
val delta_bindings : nsenv -> nsenv -> binding_table

(* Returns all of the bindings *)

val flatten_bindings : nsenv -> binding_table
val cleanup_bindings : binding_table -> binding_table -> binding_table

(* Are the nsenvs the identical ?*)

val same_nsenv : nsenv -> nsenv -> bool

