(***********************************************************************)
(*                                                                     *)
(*                                 GALAX                               *)
(*                              XQuery Engine                          *)
(*                                                                     *)
(*  Copyright 2001-2003, Lucent Technologies.                          *)
(*  Distributed only by permission.                                    *)
(*                                                                     *)
(***********************************************************************)

(* $Id: norm_context.mli,v 1.14 2005/11/22 13:47:03 mff Exp $ *)

(* Module Norm_context
   Description:
     This module implements the part of the context which is required
     during the normalization phase.
*)

open Processing_context

open Xquery_common_ast
open Xquery_core_ast

(* Type for the normalization context *)

type norm_context 

(* Build a normalization context *)


val build_norm_context :
    processing_context ->
      Namespace_context.nsenv ->
	Xquery_type_core_ast.cxschema ->
	    ((cfname * int) * (cfname * cfunction_signature)) list -> norm_context

(* Default normalization context *)

val default_norm_context : Processing_context.processing_context  -> norm_context

(* Replace the processing context *)

val replace_processing_context_in_norm_context :
    processing_context -> norm_context -> norm_context

val replace_namespace_env_in_norm_context :
    Namespace_context.nsenv -> norm_context -> norm_context

(* Copy the entire norm context *)

val copy_norm_context : norm_context -> norm_context

(* Create copy of norm context, preserving the processing context and
   function signatures, but using a new namespace and schema envs. *)

val copy_norm_context_with_sigs : norm_context -> Namespace_context.nsenv -> Xquery_type_core_ast.cxschema -> norm_context

(* Add a list of function signatures to normalization context *)

val add_sigs_to_norm_context :
    norm_context -> ((cfname * int) * (cfname * cfunction_signature)) list -> norm_context

(* Extract components from normalization the context *)

val processing_context_from_norm_context : norm_context -> Processing_context.processing_context
val nsenv_from_norm_context  		 : norm_context -> Namespace_context.nsenv
val cxschema_from_norm_context 		 : norm_context -> Xquery_type_core_ast.cxschema

(* Lookup a function's signatures in normalization context *)

val one_sig_from_norm_context : norm_context -> (cfname * int) -> cfunction_signature


