(***********************************************************************)
(*                                                                     *)
(*                                 GALAX                               *)
(*                              XQuery Engine                          *)
(*                                                                     *)
(*  Copyright 2001-2003, Lucent Technologies.                          *)
(*  Distributed only by permission.                                    *)
(*                                                                     *)
(***********************************************************************)

open Algebra_type
open Xquery_algebra_ast
open Xquery_common_ast

type ('a,'b) logical_op = ('a, 'b) Xquery_algebra_ast.aalgop_expr 

val tuple_independent   : ('a,'b) logical_op -> ('a,'b) logical_op -> bool

(* These may move to AST utils *)
val is_access_tuple       : ('a,'b) logical_op -> bool

val is_let                : ('a,'b) logical_op -> bool
val is_any_join           : ('a,'b) logical_op -> bool
val is_outer_join         : ('a,'b) logical_op -> bool
val is_regular_join       : ('a,'b) logical_op -> bool

val is_a_tuple_input_map  : ('a,'b) logical_op -> bool
val is_a_sep_sequence     : ('a,'b) logical_op -> bool
val is_a_map_index        : ('a,'b) logical_op -> bool
val is_a_dep_map          : ('a,'b) logical_op -> bool
val is_a_map              : ('a,'b) logical_op -> bool
val is_an_outer_map       : ('a,'b) logical_op -> bool
val is_a_non_concat_map   : ('a,'b) logical_op -> bool
val is_a_map_concat       : ('a,'b) logical_op -> bool
val is_any_map_concat     : ('a,'b) logical_op -> bool
val is_a_map_from_item    : ('a,'b) logical_op -> bool
val is_a_map_to_item      : ('a,'b) logical_op -> bool
val is_select             : ('a,'b) logical_op -> bool
val is_product            : ('a,'b) logical_op -> bool
val is_empty_tuple        : ('a,'b) logical_op -> bool
val is_some               : ('a,'b) logical_op -> bool
val is_null_map           : ('a,'b) logical_op -> bool
(* Added to remove materialize in all cases except updates *)
val is_a_materialize	  : ('a,'b) logical_op -> bool
(* Function matches *)
val is_boolean            : ('a,'b) logical_op -> bool
val is_distinct_value     : ('a,'b) logical_op -> bool
val is_fn_data            : ('a,'b) logical_op -> bool
val is_equal              : ('a,'b) logical_op -> bool
val is_gt                 : ('a,'b) logical_op -> bool
val is_ge                 : ('a,'b) logical_op -> bool
val is_lt                 : ('a,'b) logical_op -> bool
val is_le                 : ('a,'b) logical_op -> bool
val is_fs_untyped_to_any  : ('a,'b) logical_op -> bool
val is_input_tuple        : ('a,'b) logical_op -> bool
val is_group              : ('a,'b) logical_op -> bool

val get_let_name               : ('a,'b) logical_op -> crname
val get_map_from_item_name     : ('a,'b) logical_op -> crname
val get_outer_map_name         : ('a,'b) logical_op -> crname
val get_sep_sequence_name      : ('a,'b) logical_op -> crname
val get_outer_join_name        : ('a,'b) logical_op -> crname
val get_null_map_name          : ('a,'b) logical_op -> crname
val get_function_name          : ('a,'b) logical_op -> crname option

(* This should go somewhere else *)
type sub_expr_kind = 
  | Dependent 
  | Independent

type ('a, 'b) child_desc = ('a, 'b) aalgop_expr * sub_expr_kind * int
(* type 'a alg_child_desc  =  ('a option, unit) child_desc *)

val get_sub_expr  : ('a,'b) aalgop_expr -> sub_expr_kind -> ('a,'b) aalgop_sub_exprs
val make_child_desc : ('a,'b) aalgop_expr -> sub_expr_kind -> int -> ('a,'b) child_desc
val update_parent   : ('a,'b) child_desc option -> ('a,'b) aalgop_expr  -> unit
val get_op_from_sub_expr : ('a,'b) aalgop_expr -> sub_expr_kind -> int -> ('a,'b) aalgop_expr

