(***********************************************************************)
(*                                                                     *)
(*                                 GALAX                               *)
(*                              XQuery Engine                          *)
(*                                                                     *)
(*  Copyright 2001-2003, Lucent Technologies.                          *)
(*  Distributed only by permission.                                    *)
(*                                                                     *)
(***********************************************************************)

(* $Id: parse_top.ml,v 1.15 2005/11/16 19:54:46 simeon Exp $ *)

(* Module: Parse_top
   Description:
     This module implements top-level parsing functions.
 *)

open Finfo
open Error
open Encoding

open Xquery_common_ast
open Xquery_ast

open Lexing_util
open Parse_util


(*******************************)
(* Top-level parsing functions *)
(*******************************)

(* Statement *)

let parse_statement_from_io parse_ctxt gio =
  let proc_ctxt = Parse_context.get_processing_context parse_ctxt in
  let lh = init_xquery_lexing () in
  let gin = Parse_io.galax_utf8_input_from_input_spec proc_ctxt gio in
  try
    begin
      let lexbuf = Parse_io.lexbuf_from_galax_input gin in
      try
	let ast = Parse_xquery.statement (xquery_lexfun lh) lexbuf in
	begin
	  Parse_io.close_galax_input gin;
	  ast
	end
      with
      | Parsing.Parse_error ->
	  let finfo = (make_finfo (Lexing.lexeme_start lexbuf) (Lexing.lexeme_end lexbuf)) in
	  let msg = Parse_io.parse_error_msg gio in
	  raise (Query(Parsing (finfo, msg)))
    end
  with e ->
    Parse_io.close_galax_input gin;
    raise e

(* Prolog *)

let parse_prolog_from_io parse_ctxt gio =
  let proc_ctxt = Parse_context.get_processing_context parse_ctxt in
  let lh = init_xquery_lexing () in
  let gin = Parse_io.galax_utf8_input_from_input_spec proc_ctxt gio in
  try
    let lexbuf = Parse_io.lexbuf_from_galax_input gin in
    try
      let ast = Parse_xquery.prolog (xquery_lexfun lh) lexbuf in
      begin
	Parse_io.close_galax_input gin;
	(parse_ctxt,ast)
      end
    with Parsing.Parse_error ->
      let finfo = (make_finfo (Lexing.lexeme_start lexbuf) (Lexing.lexeme_end lexbuf)) in
      let msg = Parse_io.parse_error_msg gio in
      raise (Query(Parsing (finfo, msg)))
  with e ->
    Parse_io.close_galax_input gin;
    raise e

(* Library module *)

let parse_library_module_from_io parse_ctxt gio =
  let proc_ctxt = Parse_context.get_processing_context parse_ctxt in
  let lh = init_xquery_lexing () in
  let gin = Parse_io.galax_utf8_input_from_input_spec proc_ctxt gio in
  try
    let lexbuf = Parse_io.lexbuf_from_galax_input gin in
    try
      let ast = Parse_xquery.librarymodule (xquery_lexfun lh) lexbuf in
      begin
	Parse_io.close_galax_input gin;
	(parse_ctxt,ast)
      end
    with Parsing.Parse_error ->
(* print_string("In parse_library_module_from_io : Parse_error\n"); *)
      let finfo = (make_finfo (Lexing.lexeme_start lexbuf) (Lexing.lexeme_end lexbuf)) in
      let msg = Parse_io.parse_error_msg gio in
      raise (Query(Parsing (finfo, msg)))
  with e ->
(* printf_error "" e;  *)
    Parse_io.close_galax_input gin;
    raise e

(* Main module *)

let parse_main_module_from_io parse_ctxt gio =
  let proc_ctxt = Parse_context.get_processing_context parse_ctxt in
  let lh = init_xquery_lexing () in
  let gin = Parse_io.galax_utf8_input_from_input_spec proc_ctxt gio in
  try
    let lexbuf = Parse_io.lexbuf_from_galax_input gin in
    try
      let ast = Parse_xquery.mainmodule (xquery_lexfun lh) lexbuf in
      begin
	Parse_io.close_galax_input gin;
	(parse_ctxt,ast)
      end
    with Parsing.Parse_error -> 
      let finfo =
	(make_finfo (Lexing.lexeme_start lexbuf) (Lexing.lexeme_end lexbuf))
      in
      let msg = Parse_io.parse_error_msg gio in
      raise (Query(Parsing (finfo, msg)))
  with e ->
    Parse_io.close_galax_input gin;
    raise e


(* Content models *)

let parse_type_from_io parse_ctxt gio =
  let proc_ctxt = Parse_context.get_processing_context parse_ctxt in
  let lh = init_type_lexing () in
  let gin = Parse_io.galax_utf8_input_from_input_spec proc_ctxt gio in
  try
    let lexbuf = Parse_io.lexbuf_from_galax_input gin in
    try
      let ast = Parse_xquery.extype (xquery_lexfun lh) lexbuf in
      begin
	Parse_io.close_galax_input gin;
	ast
      end
    with Parsing.Parse_error -> 
      let finfo = (make_finfo (Lexing.lexeme_start lexbuf) (Lexing.lexeme_end lexbuf)) in
      let msg = Parse_io.parse_error_msg gio in
      raise (Query(Parsing (finfo, msg)))
  with e ->
(* printf_error "" e;  *)
    Parse_io.close_galax_input gin;
    raise e

