(***********************************************************************)
(*                                                                     *)
(*                                 GALAX                               *)
(*                              XQuery Engine                          *)
(*                                                                     *)
(*  Copyright 2001-2004, Lucent Technologies and AT&T Labs.            *)
(*  Distributed only by permission.                                    *)
(*                                                                     *)
(***********************************************************************)

(* Module: Algebgra_print_common
   Description:

   This module contains common data for the parser and the printer

   NOTE: The parser is not currently hooked into the system
   NOTE: This code does not currently support the serialization of updates

*)
open Namespace_names

val algebra_prefix : prefix

val algebra_uri    : uri 

val algebra_bindings : Namespace_context.binding_table

val alg_elem         : rqname -> Namespace_symbols.relem_symbol
val alg_attr         : rqname -> Namespace_symbols.rattr_symbol

val string_of_relem_symbol: Namespace_symbols.relem_symbol -> string
val string_of_rattr_symbol: Namespace_symbols.rattr_symbol -> string
val string_of_rtype_symbol: Namespace_symbols.rtype_symbol -> string


val function_tag_elem_name  : rqname
val function_body_elem_name : rqname

val input_datamodel_elem_name : rqname
val output_datamodel_elem_name : rqname
val datamodel_signature : rqname


val dep_elem_name   : rqname
val indep_elem_name : rqname

val sub_expression_arity_attr_name : rqname
val sub_expression_type_attr_name  : rqname

val input_type_elem_name : rqname
val output_type_elem_name : rqname
val function_signature_elem_name : rqname

val out_type_attr_name : rqname
val arg_count_attr_name : rqname
val index_attr_name : rqname
val datamodel_attr_name : rqname
(* Put in a sample format of the prolog *)

val no_input   : string
val one_input  : string
val two_input  : string
val many_input : string

val input_prune_field_attr_name    : rqname
val input_distinct_field_attr_name : rqname

(* Axis Related *)
val node_test_elem_name : rqname

val name_test_attr_name : rqname
val name_test_elem_name : rqname

val kind_test_elem_name : rqname
val kind_test_attr_name : rqname

val pi_kind_test_elem_name : rqname
val pi_kind_attr_name : rqname
val axis_attr_name : rqname

(* Typeswitch Element *)
val ts_case_elem_name : rqname
val ts_default_elem_name : rqname
val ts_num_branches_attr_name : rqname
(* Tuple element *)
val tuple_slot_elem_name : rqname

val var_attr_name : rqname
val type_attr_name : rqname

val vname_attr_name : rqname
val stablekind_attr_name : rqname
val sortkind_attr_name : rqname
val emptysortkind_attr_name : rqname
val optdatatype_attr_name : rqname
val pos_attr_name : rqname
val usage_attr_name : rqname
val value_attr_name : rqname
val atomic_type_attr_name : rqname
val target_attr_name : rqname
val content_attr_name : rqname
val attr_name_attr_name : rqname
val elem_name_attr_name : rqname
val fn_name_attr_name : rqname
val arity_attr_name : rqname
val kname_attr_name : rqname
val host_attr_name : rqname
val port_attr_name : rqname
val tuple_name_attr_name : rqname
val occurrence_attr_name : rqname

val prefix_attr_name : rqname
val rqname_attr_name : rqname

val relem_elem_name : rqname
val rattr_elem_name : rqname
val rtype_elem_name : rqname

val asequencetype_elem_name : rqname
(* ait element names *)
val ait_attributetype_elem_name : rqname
val ait_elementtype_elem_name : rqname
val ait_schemaattribute_elem_name : rqname
val ait_schemaelementtype_elem_name : rqname
val ait_kindtest_elem_name : rqname
val ait_typeref_elem_name : rqname
val ait_node_elem_name : rqname
val ait_item_elem_name : rqname
val ait_numeric_elem_name : rqname
val ait_text_elem_name : rqname
val ait_comment_elem_name : rqname
val ait_processing_instruction_elem_name : rqname
val ait_empty_elem_name : rqname
val ait_anyatomic_elem_name : rqname
val ait_atomic_elem_name : rqname
val ait_document_elem_name : rqname

(* ait attribute names *)
val schema_attr_name_attr_name : rqname
val schema_elem_name_attr_name : rqname
val type_ref_attr_name : rqname
val pi_arg_attr_name : rqname
val atomic_type_attr_name : rqname

val function_decl_elem_name : rqname
val fn_count_attr_name : rqname

val variable_declarations_elem_name : rqname
val variable_decl_elem_name         : rqname
val variable_count_attr_name        : rqname
val variable_name_attr_name         : rqname

val null_index_attr_name : rqname
val input_dot_attr_name  : rqname
val output_dot_attr_name : rqname

val index_definitions_elem_name : rqname
val index_def_elem_name     	: rqname
val op_1_elem_name    	    	: rqname
val op_2_elem_name    	    	: rqname
val kname_attr_name   	    	: rqname
val indices_count_attr_name 	: rqname

val statement_declaration_elem_name : rqname
val expression_elem_name : rqname
val number_of_statements_attr_name : rqname

val algop_module_decl_elem_name : rqname
val prolog_elem_name: rqname
val attribute_of_prolog_algop_name: Xquery_algebra_ast.algop_decl_name -> rqname * string



val insert_location_attr_name   : rqname
val value_of_flag_attr_name     : rqname  
val snap_modifier_attr_name     : rqname
val docname_attr_name           : rqname

val project_name_elem_name      : rqname
val project_name_attr_name      : rqname
val project_elem_name           : rqname
val item_tuple_attr_name        : rqname

val n_sort_criteria_attr_name   : rqname
val stable_attr_name            : rqname
val sort_spec_elem_name         : rqname
val empty_sort_kind_attr_name   : rqname
val sort_kind_attr_name         : rqname

(**********************)
(* Compile Annotation *)
(**********************)

val compile_annotation_elem_name  : rqname
val accessed_fields_elem_name     : rqname
val returned_fields_elem_name     : rqname
val free_variables_elem_name      : rqname
    (* Attributes *)
val num_accessed_fields_attr_name : rqname
val num_returned_fields_attr_name : rqname
val num_free_variables_attr_name  : rqname
(***********************)
(* Overloaded Function *)
(***********************)

val overloaded_call_elem_name     : rqname



(***********************)
(* Grouping operations *)
(***********************)
val vname_elem_name          : rqname
val gbn_elem_name            : rqname
val induced_elem_name        : rqname
val mbv_elem_name            : rqname
val agg_elem_name            : rqname
val group_desc_elem_name     : rqname
val distinct_value_attr_name : rqname

(***********************)
(* TupleTreePatterns   *)
(***********************)
val ttp_name            : rqname
val twig_node_name      : rqname
val in_field_attr_name  : rqname
val restore_attr_name   : rqname
val out_field_attr_name : rqname
val child_twig_name     : rqname
val pred_twig_name      : rqname

(**************)
(* Predicates *)
(**************)
val simple_conjunct_elem_name       : rqname
val complex_conjunct_elem_name      : rqname
val disjunct_elem_name              : rqname
val simple_conjunct_start_attr_name : rqname
val simple_conjunct_end_attr_name   : rqname

(**********)
(* Prolog *)
(**********)

type prolog_algop_moniker = 
    AOEVarDeclExternal_n 
  | AOEVarDecl_n
  | AOEValueIndexDecl_n 
  | AOENameIndexDecl_n 

val prolog_attr_name             : rqname
val prolog_var_count_attr_name   : rqname
val prolog_index_count_attr_name : rqname
val prolog_vars_elem_name        : rqname
val prolog_indices_elem_name     : rqname
val get_prolog_algop_moniker     : string -> prolog_algop_moniker
val algop_prolog_decl_elem_name  : rqname

type algop_moniker = 
  | AOELet_n
  | AOEIf_n
  | AOETypeswitch_n
  | AOEVar_n
  | AOEScalar_n
  | AOESeq_n
  | AOEEmpty_n
  | AOEDocument_n
  | AOEPI_n
  | AOEPIComputed_n
  | AOEComment_n
  | AOECommentComputed_n
  | AOEText_n
  | AOETextComputed_n
  | AOEElem_n
  | AOEAnyElem_n
  | AOEAttr_n
  | AOEAnyAttr_n
  | AOEError_n
  | AOETreat_n
  | AOEValidate_n
  | AOECast_n
  | AOECastable_n
  | AOESome_n 
  | AOEEvery_n

  | AOETreeJoin_n
  | AOETupleTreePattern_n
  | AOEProject_n
  | AOEPrune_n
  | AOEDistinct_n

  (* Function Calls *)
  | AOECallBuiltIn_n
  | AOECallOverloaded_n 
  | AOECallUserDefined_n
  | AOEConvertSimple_n
  | AOEPromoteNumeric_n
  | AOEUnsafePromoteNumeric_n

  | AOEKeyRef_n
  | AOEExecute_n

  (* Tuples *)
  | AOECreateTuple_n          (* Creates a new tuple *)
  | AOEAccessTuple_n          (* Accesses a new tuple *)
  | AOEOrderBy_n
  | AOEMaterializeTable_n
  | AOEProduct_n
  | AOESelect_n
  | AOEJoin_n
  | AOELeftOuterJoin_n
  | AOEConcatTuples_n         (* Tuple concatenation *)
  | AOEMapFromItem_n          (* Item to tuple iteration *)
  | AOEMapToItem_n            (* Tuple to item iteration *)
  | AOEMap_n                  (* Tuple iteration *)
  | AOEMapIndex_n
  | AOEMapIndexStep_n
  | AOEMapConcat_n
  | AOEOuterMapConcat_n
  | AOEInputTuple_n
  | AOEGroupBy_n
  | AOENullMap_n

  (* Updates *)
  | AOESnap_n
  | AOEDelete_n
  | AOEDetach_n
  | AOEInsert_n
  | AOERename_n
  | AOEReplace_n
  | AOESequencing_n 
  | AOEParse_n

val make_input_attr_name : int -> rqname


val get_serializable_moniker_of_algop_name : Xquery_algebra_ast.algop_expr_name -> rqname
val get_moniker_of_serialized_moniker     : Namespace_symbols.relem_symbol -> algop_moniker
