(***********************************************************************)
(*                                                                     *)
(*                               Galax                                 *)
(*                            XQuery Engine                            *)
(*                                                                     *)
(*  Copyright 2001-2003, Lucent Technologies.                          *)
(*  Distributed only by permission.                                    *)
(*                                                                     *)
(***********************************************************************)

(* $Id: galax-test.ml,v 1.19 2006/03/02 17:40:52 mff Exp $ *)

(* Module: Galax-test
   Description:
     The galax-test command can be used to process tests following the
     W3C XML Query test suite format.
 *)

open Galax
open Galax_io

open Error
open Processing_context

open Datatypes
open Dm_atomic
open Dm

open Streaming_types
open Streaming_util
open Streaming_ops
open Streaming_constructors
open Streaming_diff

open Top_options
open Test_core


(********)
(* Main *)
(********)

let run_unit all_bugs unit =
  begin
    make_catalog unit ();
    Conf.warning := false;
    let all_groups = run_groups unit all_bugs in
    get_all_results all_groups
  end

let run_tests config () =
  try
    let all_bugs = get_all_bugs config in
    let result_file = config.result_file in
    let all = List.map (fun (name,unit) -> (name,!(unit.unit_version),run_unit all_bugs unit)) config.units in
    Printf.printf "Producing result file...";flush stdout;
    let r1 = total_result all in
    let oc = open_out result_file in
    begin
      serialize_as_sax proc_ctxt (Channel_Output oc) r1;
      Printf.printf "done.\n\n";flush stdout;
      close_out oc
    end
  with
  | exn -> Error.printf_error "ERROR!" exn 

let usage_galax_test = "Usage: %s configfile"

let _ =
  try
    let args =
      make_options
	proc_ctxt
	usage_galax_test
	[ Misc_Options;Encoding_Options;DataModel_Options;Behavior_Options;Optimization_Options;CodeSelection_Options;Runtime_Options]
    in
    let file =
      match args with
      | [] -> failwith "Input file(s) not specified"
      | [fname] ->
	  fname
      | _ -> failwith "Cannot have several config files"
    in
    Printf.printf "\n\n";
    Printf.printf "Running XQuery test suite\n";
    Printf.printf "=========================\n\n";
    let config = top_make_test_config file in
    run_tests config ()
  with
  | e ->
      begin
	eprintf_error "  " e;
	Format.fprintf (!Conf.glx_err_formatter) "@."
      end


