(***********************************************************************)
(*                                                                     *)
(*                                 GALAX                               *)
(*                              XQuery Engine                          *)
(*                                                                     *)
(*  Copyright 2001-2004, Lucent Technologies.                          *)
(*  Distributed only by permission.                                    *)
(*                                                                     *)
(***********************************************************************)

(* $Id: rewriting_ddo_util.ml,v 1.1 2005/04/14 16:41:08 simeon Exp $ *)

(* Module: Rewriting_ddo_util
   Description:
     This module implements utilities used during DDO rewriting.
*)

open Hashtbl
open Rewriting_ddo_automaton
open Processing_context

open Namespace_names

(* replace true with Conf.debugging *)
let debug message =
  if !Conf.debugging then 
    Printf.printf "In module DDO_SYMBOLTABLE: %s \n" message

type st = ((prefix * uri * string), state) Hashtbl.t

let create_symbol_table () = 
  Hashtbl.create 100

(**
  Hashtbl.add tbl x y adds a binding of x to y in table tbl. Previous bindings 
  for x are not removed, but simply hidden. That is, after performing 
  Hashtbl.remove tbl x, the previous binding for x, if any, is restored. 
  (Same behavior as with association lists.) **)
let register_var symtab (prefix, uri_name, name) st =
  debug (String.concat "" ["Adding symbol:"; name; " -> "; (print_state st)]);
  Hashtbl.add symtab (prefix, uri_name, name) st

(** restore previous binding of name (if any) **)
let unregister_var symtab var =
  Hashtbl.remove symtab var

(** get the state for $name **)
let get_state sbdo symtab (prefix, uri_name, name) =
    try 
      let st = Hashtbl.find symtab (prefix, uri_name, name) in
        debug (String.concat "" ["Looking up symbol : "; name; " -> "; (print_state st)]); 
	st
    with Not_found ->
      sink_state sbdo

