(***********************************************************************)
(*                                                                     *)
(*                                 GALAX                               *)
(*                              XQuery Engine                          *)
(*                                                                     *)
(*  Copyright 2001-2003, Lucent Technologies.                          *)
(*  Distributed only by permission.                                    *)
(*                                                                     *)
(***********************************************************************)

(* $Id: prefix_context.ml,v 1.2 2004/08/25 15:42:01 simeon Exp $ *)

(* Module: Prefix_context
   Description:
     This module implements the context necessary to turn a resolved
     SAX stream back into an unresolved one with the proper xmlns
     attributes.
*)


(*********************************)
(* A type for the prefix context *)
(*********************************)

type prefix_context =
    { prefix_context_nsenv : Namespace_context.nsenv Stack.t }


(********************************)
(* Creates a new prefix context *)
(********************************)

let build_prefix_context () =
    { prefix_context_nsenv = Stack.create () }



(*********************************)
(* Operations on prefix contexts *)
(*********************************)

let push_nsenv_in_prefix_context prefix_context new_nsenv =
  let delta_bindings =
    try
      let previous_nsenv = Stack.top prefix_context.prefix_context_nsenv in
      if Namespace_context.same_nsenv new_nsenv previous_nsenv
      then []
      else
	Namespace_context.delta_bindings new_nsenv previous_nsenv
    with
    | Stack.Empty ->
	Namespace_context.flatten_bindings new_nsenv
  in
  Stack.push new_nsenv prefix_context.prefix_context_nsenv;
  delta_bindings

let pop_nsenv_from_prefix_context prefix_context =
  ignore(Stack.pop prefix_context.prefix_context_nsenv)

