val server_error_msg : exn -> string

module Sim :
  sig
    val get_files_in_directory : string -> string list
    val convert_regexp : string -> string
    val ls : string -> string -> string list
    val string_of_file : string -> string
    val portmap : string -> int -> (string * int * string) list
  end
module Gui :
  sig
    type nodeName = string
    type link = int * int * int * string
    type guiEvent =
        GuiMessage of nodeName * nodeName * string * string * link list
      | GuiNode of nodeName * string * string * link list
      | GuiNewNode of nodeName * string * link list
      | GuiNone
    val guievent_of_string : string -> guiEvent
    val string_of_guievent : guiEvent -> string
    val gui_report : guiEvent -> unit
  end

type xquery_kind = 
    XQueryString
  | XQueryPlan
module Server :
  sig
    val debug : bool ref 
    val default_port : int
    val pm : (Gui.nodeName * int * string) list ref
    exception MyError of string
    val full_hostname : string
    val short_hostname : string
    val my_host_port : Gui.nodeName ref
    val do_query : Gui.nodeName * string option * xquery_kind * string -> string
    val message : string -> unit
    val http_protect : string -> string
    val server :
      (string -> xquery_kind -> string -> string) ->
      string -> in_channel -> out_channel -> unit
    val go : int -> (string -> xquery_kind -> string -> string) -> string -> unit
  end

