(***********************************************************************)
(*                                                                     *)
(*                               Galax                                 *)
(*                            XQuery Engine                            *)
(*                                                                     *)
(*  Copyright 2001-2003, Lucent Technologies.                          *)
(*  Distributed only by permission.                                    *)
(*                                                                     *)
(***********************************************************************)

(* $Id: top_options.mli,v 1.25 2006/05/08 12:09:39 simeon Exp $ *)

(* Module: Top_options
   Description:
     This module contains code and messages for command-line options.
 *)

(*******************)
(* Version message *)
(*******************)

val print_version : unit -> unit


(***********)
(* Options *)
(***********)

type option_classes =
  | GalaxParse_Options
  | Misc_Options
  | Monitoring_Options
  | Encoding_Options
  | Context_Options
  | DataModel_Options
  | Serialization_Options
  | Behavior_Options
  | ProcessingPhases_Options
  | Printing_Options
  | Optimization_Options
  | CodeSelection_Options
  | Runtime_Options
  | PrintParse_Options

(* Misc options *)

val opt_version : string
val arg_version : Processing_context.processing_context -> unit -> unit
val msg_version : string

val opt_verbose : string
val arg_verbose : Processing_context.processing_context -> string -> unit
val msg_verbose : string

val opt_debug : string
val arg_debug : Processing_context.processing_context -> bool ref
val msg_debug : string

val opt_streaming_shebang : string
val arg_streaming_shebang : Processing_context.processing_context -> string -> unit
val msg_streaming_shebang : string

val opt_scjoin_shebang : string
val arg_scjoin_shebang : Processing_context.processing_context -> string -> unit
val msg_scjoin_shebang : string

val opt_twigjoin_shebang : string
val arg_twigjoin_shebang : Processing_context.processing_context -> string -> unit
val msg_twigjoin_shebang : string

val opt_print_plan_shebang : string
val arg_print_plan_shebang : Processing_context.processing_context -> string -> unit
val msg_print_plan_shebang : string


(* Monitoring options *)

val opt_monitor : string
val arg_monitor : Processing_context.processing_context -> string -> unit
val msg_monitor : string

val opt_monitor_mem : string
val arg_monitor_mem : Processing_context.processing_context -> string -> unit
val msg_monitor_mem : string

val opt_monitor_time : string
val arg_monitor_time : Processing_context.processing_context -> string -> unit
val msg_monitor_time : string

val opt_output_monitor : string
val arg_output_monitor : Processing_context.processing_context -> string -> unit
val msg_output_monitor : string

(* Character encoding options *)

val opt_internal_encoding : string
val arg_internal_encoding : Processing_context.processing_context -> string -> unit
val msg_internal_encoding : string

val opt_output_encoding : string
val arg_output_encoding : Processing_context.processing_context -> string -> unit
val msg_output_encoding : string

(* Input context options *)

val opt_base_uri : string
val arg_base_uri : Processing_context.processing_context -> string -> unit
val msg_base_uri : string

val opt_var : string
val arg_var : Processing_context.processing_context -> string -> unit
val msg_var : string

val opt_doc : string
val arg_doc : Processing_context.processing_context -> string -> unit
val msg_doc : string

val opt_context_item : string
val arg_context_item : Processing_context.processing_context -> string -> unit
val msg_context_item : string

val opt_context : string
val arg_context : Processing_context.processing_context -> string -> unit
val msg_context : string

(* Data model options *)

val opt_xml_whitespace : string
val arg_xml_whitespace : Processing_context.processing_context -> unit -> unit
val msg_xml_whitespace : string

val opt_xml_pic : string
val arg_xml_pic : Processing_context.processing_context -> unit -> unit
val msg_xml_pic : string

(* Serialization options *)

val opt_serialize : string
val arg_serialize : Processing_context.processing_context -> string -> unit
val msg_serialize : string

(* Behavior options *)

val opt_typing : string
val arg_typing : Processing_context.processing_context -> string -> unit
val msg_typing : string

val opt_boundary_space : string
val arg_boundary_space : Processing_context.processing_context -> string -> unit
val msg_boundary_space : string

val opt_construction : string
val arg_construction : Processing_context.processing_context -> string -> unit
val msg_construction : string

val opt_ordering : string
val arg_ordering : Processing_context.processing_context -> string -> unit
val msg_ordering : string

val opt_default_order : string
val arg_default_order : Processing_context.processing_context -> string -> unit
val msg_default_order : string

val opt_ns_preserve : string
val arg_ns_preserve : Processing_context.processing_context -> string -> unit
val msg_ns_preserve : string

val opt_ns_inherit : string
val arg_ns_inherit : Processing_context.processing_context -> string -> unit
val msg_ns_inherit : string

(* Processing phases options *)

val opt_normalize : string
val arg_normalize : Processing_context.processing_context -> string -> unit
val msg_normalize : string

val opt_static_typing : string
val arg_static_typing : Processing_context.processing_context -> string -> unit
val msg_static_typing : string

val opt_rewriting : string
val arg_rewriting : Processing_context.processing_context -> string -> unit
val msg_rewriting : string

val opt_factorization : string
val arg_factorization : Processing_context.processing_context -> string -> unit
val msg_factorization : string

val opt_optimization : string
val arg_optimization : Processing_context.processing_context -> string -> unit
val msg_optimization : string

val opt_dynamic : string
val arg_dynamic : Processing_context.processing_context -> string -> unit
val msg_dynamic : string

val opt_execute : string
val arg_execute : Processing_context.processing_context -> string -> unit
val msg_execute : string


(* Printing options *)

val opt_print_error : string
val arg_print_error : Processing_context.processing_context -> string -> unit
val msg_print_error : string

val opt_print_plan : string
val arg_print_plan : Processing_context.processing_context -> string -> unit
val msg_print_plan : string

val opt_output_err : string
val arg_output_err : Processing_context.processing_context -> string -> unit
val msg_output_err : string

val opt_print_xml : string
val arg_print_xml : Processing_context.processing_context -> string -> unit
val msg_print_xml : string

val opt_output_xml : string
val arg_output_xml : Processing_context.processing_context -> string -> unit
val msg_output_xml : string

val opt_print_type : string
val arg_print_type : Processing_context.processing_context -> string -> unit
val msg_print_type : string

val opt_output_type : string
val arg_output_type : Processing_context.processing_context -> string -> unit
val msg_output_type : string

val opt_print_expr : string
val arg_print_expr : Processing_context.processing_context -> string -> unit
val msg_print_expr : string

val opt_output_expr : string
val arg_output_expr : Processing_context.processing_context -> string -> unit
val msg_output_expr : string

val opt_print_norm_expr : string
val arg_print_norm_expr : Processing_context.processing_context -> string -> unit
val msg_print_norm_expr : string

val opt_output_norm_expr : string
val arg_output_norm_expr : Processing_context.processing_context -> string -> unit
val msg_output_norm_expr : string

val opt_print_rewr_expr : string
val arg_print_rewr_expr : Processing_context.processing_context -> string -> unit
val msg_print_rewr_expr : string

val opt_output_rewr_expr : string
val arg_output_rewr_expr : Processing_context.processing_context -> string -> unit
val msg_output_rewr_expr : string

val opt_print_fact_expr : string
val arg_print_fact_expr : Processing_context.processing_context -> string -> unit
val msg_print_fact_expr : string

val opt_output_fact_expr : string
val arg_output_fact_expr : Processing_context.processing_context -> string -> unit
val msg_output_fact_expr : string

val opt_print_global : string
val arg_print_global : Processing_context.processing_context -> string -> unit
val msg_print_global : string

val opt_print_annotations : string
val arg_print_annotations : Processing_context.processing_context -> string -> unit
val msg_print_annotations : string

val opt_print_prolog : string
val arg_print_prolog : Processing_context.processing_context -> string -> unit
val msg_print_prolog : string

val opt_output_all : string
val arg_output_all : Processing_context.processing_context -> string -> unit
val msg_output_all : string

val opt_print_log_plan : string
val arg_print_log_plan : Processing_context.processing_context -> string -> unit
val msg_print_log_plan : string

val opt_output_log_plan : string
val arg_output_log_plan : Processing_context.processing_context -> string -> unit
val msg_output_log_plan : string

val opt_print_log_rewrites : string
val arg_print_log_rewrites : Processing_context.processing_context -> string -> unit
val msg_print_log_rewrites : string

val opt_output_log_rewrites : string
val arg_output_log_rewrites : Processing_context.processing_context -> string -> unit
val msg_output_log_rewrites : string


val opt_print_opt_plan : string
val arg_print_opt_plan : Processing_context.processing_context -> string -> unit
val msg_print_opt_plan : string

val opt_output_opt_plan : string
val arg_output_opt_plan : Processing_context.processing_context -> string -> unit
val msg_output_opt_plan : string

val opt_print_phys_plan : string
val arg_print_phys_plan : Processing_context.processing_context -> string -> unit
val msg_print_phys_plan : string

val opt_output_phys_plan : string
val arg_output_phys_plan : Processing_context.processing_context -> string -> unit
val msg_output_phys_plan : string

val opt_print_comp_annot : string
val arg_print_comp_annot : Processing_context.processing_context -> string -> unit
val msg_print_comp_annot : string

val opt_print_materialize : string
val arg_print_materialize : Processing_context.processing_context -> string -> unit
val msg_print_materialize : string

val opt_output_dfgraph : string
val arg_output_dfgraph : Processing_context.processing_context -> string -> unit
val msg_output_dfgraph : string


(* Optimization options *)

val opt_aggressive : string
val arg_aggressive : Processing_context.processing_context -> string -> unit
val msg_aggressive : string

val opt_projection : string
val arg_projection : Processing_context.processing_context -> string -> unit
val msg_projection : string

val opt_treejoin_log : string
val arg_treejoin_log : Processing_context.processing_context -> string -> unit
val msg_treejoin_log : string

val opt_treejoin_phys : string
val arg_treejoin_phys : Processing_context.processing_context -> string -> unit
val msg_treejoin_phys : string

val opt_pruning : string
val arg_pruning : Processing_context.processing_context -> string -> unit
val msg_pruning : string

val opt_streaming : string
val arg_streaming : Processing_context.processing_context -> string -> unit
val msg_streaming : string

val opt_code_selection_by_physical_type : string
val arg_code_selection_by_physical_type : Processing_context.processing_context -> string -> unit
val msg_code_selection_by_physical_type : string

val opt_buffer_chunks : string
val arg_buffer_chunks : Processing_context.processing_context -> string -> unit
val msg_buffer_chunks : string

val opt_buffer_csize : string
val arg_buffer_csize : Processing_context.processing_context -> string -> unit
val msg_buffer_csize : string

val opt_buffer_inc : string
val arg_buffer_inc : Processing_context.processing_context -> string -> unit
val msg_buffer_inc : string

val opt_nested_loop_join : string
val arg_nested_loop_join : Processing_context.processing_context -> string -> unit
val msg_nested_loop_join : string

val opt_sbdo : string
val arg_sbdo : Processing_context.processing_context -> string -> unit
val msg_sbdo : string

val opt_force_materialized : string
val arg_force_materialized : Processing_context.processing_context -> string -> unit
val msg_force_materialized : string

val opt_allow_streamed_variables : string
val arg_allow_streamed_variables : Processing_context.processing_context -> string -> unit
val msg_allow_streamed_variables : string

val opt_allow_streamed_tuple_fields : string
val arg_allow_streamed_tuple_fields : Processing_context.processing_context -> string -> unit
val msg_allow_streamed_tuple_fields : string

val opt_descendant_hack : string
val arg_descendant_hack : Processing_context.processing_context -> string -> unit
val msg_descendant_hack : string


(******************************)
(* Titles for option clusters *)
(******************************)

val title_main : string

val title_misc_options               : string
val title_monitor_options            : string
val title_character_encoding_options : string
val title_input_context_options      : string
val title_data_model_options         : string
val title_serialization_options      : string
val title_behavior_options           : string
val title_processing_phases_options  : string
val title_printing_options           : string
val title_prototype_options          : string
val title_optimization_options       : string
val title_code_selection_options     : string
val title_runtime_options            : string

val title_galax_parse_options        : string

(*****************)
(* Usage message *)
(*****************)

val usage_galax_run     : string
val usage_galax_parse   : string
val usage_galax_compile : string


(* Build option operations *)

val make_options : Processing_context.processing_context -> string -> option_classes list -> string list

