(***********************************************************************)
(*                                                                     *)
(*                               Galax                                 *)
(*                            XQuery Engine                            *)
(*                                                                     *)
(*  Copyright 2001-2003, Lucent Technologies.                          *)
(*  Distributed only by permission.                                    *)
(*                                                                     *)
(***********************************************************************)

(* $Id: top_util.mli,v 1.6 2005/01/26 17:04:31 simeon Exp $ *)

(* Module: Top_util
   Description:
     This module implements some support functions for Galax
     command-line tools.
 *)

(************)
(* Printing *)
(************)

val separator : string ref
val print_processing_file : string -> unit


(*********************************)
(* Argument Processing functions *)
(*********************************)

val init_output_refs  : string -> out_channel ref -> Format.formatter ref -> unit
val close_channel_ref : out_channel ref -> unit


(***********************)
(* Top-level execution *)
(***********************)

val exec :
    (Processing_context.processing_context -> 'a -> unit) ->
      Processing_context.processing_context -> 'a -> unit
val low_exec :
    (unit -> unit) -> unit -> unit


(******************)
(* Initialization *)
(******************)

val galax_run_proc_ctxt     : unit -> Processing_context.processing_context
val galax_compile_proc_ctxt : unit -> Processing_context.processing_context

val init_all : Processing_context.processing_context -> Galax.compiled_prolog


(*******************************)
(* Set up the external context *)
(*******************************)

val set_up_external_context :
    Processing_context.processing_context -> (bool * Galax.external_context)

val compile_main_module_helper :
    bool -> Galax.compiled_prolog -> Galax_io.input_spec -> Galax.compiled_prolog * (Galax.compiled_statement list)


